import os
from fastapi import APIRouter
from pydantic import BaseModel
import azure.cosmos.exceptions as exceptions
from sendgrid.helpers.mail import Mail
from sendgrid import SendGridAPIClient
from ..database.mycosmosdb import callConnect
from ..configs.config import sendgrid_email, sendgrid_id, sendgrid_id2, sendgrid_id3, sendgrid_id4, sendgrid_id5, sendgrid_key, NODE_DEV
from typing import List
import logging
from sendgrid.helpers.mail import Mail, Email, To, Content



NODE_DEV = os.getenv("NODE_DEV")

if NODE_DEV == "production":
    MY_DOMAIN = "https://metricplatformapi.com/"
    MY_HOME = "https://www.metric-platform.com/"
elif NODE_DEV == "test":
    MY_DOMAIN = "https://metricplatform-staging.azurewebsites.net/"
    MY_HOME = "https://metricplatform-info-metric-esgc-metric-dev.vercel.app/"
else:
    MY_DOMAIN = "http://localhost:8000/"
    MY_HOME = "http://localhost:3000/"




FROM_EMAIL = sendgrid_email
TEMPLATE_ID = sendgrid_id
TEMPLATE_ID2 = sendgrid_id2
TEMPLATE_ID3 = sendgrid_id3
TEMPLATE_ID4 = sendgrid_id4
TEMPLATE_ID5 = sendgrid_id5
SENDGRID_API_KEY = sendgrid_key



class InviteMember(BaseModel):
    oid: str
    city: str
    title: str
    email: str
    originemail: str
class InviteCompany(BaseModel):
    oid: str
    city: str
    title: str
    fund: str
    fundemail: str
    companyemail: str
    firm: str
class InviteCompany2(BaseModel):
    title: str
    fund: str
    email: str
    firm: str
class InviteCompany3(BaseModel):
    fund: str
    title: str


def read_item(container, doc_id, account_number):
    response = container.read_item(item=doc_id, partition_key=account_number)
    return response



emailroute = APIRouter()



@emailroute.post('/sendsignup')
async def signup_invite(passport: InviteMember):
    try:
        db_result = read_item(callConnect.container, passport.oid, passport.city)
        
    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /sendsignup container')
        return {"url": "/", "atype": False}


    try:
        db_result2 = read_item(callConnect.container5, passport.oid, passport.oid)
        
    except exceptions.CosmosHttpResponseError as e:
        try:
            current_user = read_item(callConnect.container13, passport.email, passport.email)
            user = {
                "id": passport.email,
                "oid": passport.oid,
                "team": current_user["team"]
            }
            existingUsers = current_user["team"]
            newMember = {
                "oid": passport.oid,
                "city": passport.city,
                "email": passport.originemail,
                "cname": passport.title
            }
            existingUsers.append(newMember)
            target = {
                "team": existingUsers
            }
            user.update(target)
            callConnect.container13.upsert_item(body=user)

            user2 = {
                "id": passport.oid,
                "members": [{"name": passport.email}]
            }
            callConnect.container5.upsert_item(body=user2)

            TO_EMAILS = [(passport.email, passport.title)]
            message = Mail(from_email=FROM_EMAIL, to_emails=TO_EMAILS)
            message.dynamic_template_data = {
                'fund': passport.title,
            }
            message.template_id = TEMPLATE_ID3
            
            try:
                sg = SendGridAPIClient(SENDGRID_API_KEY)
                response = sg.send(message)
                code, body, headers = response.status_code, response.body, response.headers
                
            except Exception as e:
                logging.error('This is an error message for /sendsignup container5 Error: {0}.format(e)')
                
            return {"url": f"{MY_HOME}dashboard/settings"}
        
        except exceptions.CosmosHttpResponseError as e:
            user = {
                "id": passport.email,
                "team": "yes",
                "email": db_result["email"],
                "oid": db_result["id"],
                "city": db_result["city"],
                "session_id": db_result["session_id"],
                "plan_id": db_result["plan_id"]
            }
            callConnect.container5.upsert_item(body=user)
            user2 = {
                "id": passport.oid,
                "members": [{"name": passport.email}]
            }
            callConnect.container5.upsert_item(body=user2)

            TO_EMAILS = [(passport.email, passport.title)]
            message = Mail(from_email=FROM_EMAIL, to_emails=TO_EMAILS)
            message.dynamic_template_data = {
                'fund': passport.title,
            }
            message.template_id = TEMPLATE_ID3
            
            try:
                sg = SendGridAPIClient(SENDGRID_API_KEY)
                response = sg.send(message)
                code, body, headers = response.status_code, response.body, response.headers
                
            except Exception as e:
                logging.error('This is an error message for /sendsignup container5 Error: {0}.format(e)')
                
            return {"url": f"{MY_HOME}dashboard/settings"}


    try:
        current_user = read_item(callConnect.container13, passport.email, passport.email)
        user = {
            "id": passport.email,
            "oid": passport.oid,
            "team": current_user["team"]
        }
        existingUsers = current_user["team"]
        newMember = {
            "oid": passport.oid,
            "city": passport.city,
            "email": passport.originemail,
            "cname": passport.title
        }
        existingUsers.append(newMember)
        target = {
            "team": existingUsers
        }
        user.update(target)
        callConnect.container13.upsert_item(body=user)

        user2 = {
            "id": db_result2["id"],
            "members": db_result2["members"]
        }
        mems = db_result2["members"]

        mems.append({"name": passport.email})
        project = {
            "members": mems
        }
        user2.update(project)
        callConnect.container5.upsert_item(body=user2)
        TO_EMAILS = [(passport.email, passport.title)]
        message = Mail(from_email=FROM_EMAIL, to_emails=TO_EMAILS)
        message.dynamic_template_data = {
            'fund': passport.title,
        }
        message.template_id = TEMPLATE_ID3
        
        try:
            sg = SendGridAPIClient(SENDGRID_API_KEY)
            response = sg.send(message)
            code, body, headers = response.status_code, response.body, response.headers
            
        except Exception as e:
            logging.error('This is an error message for /sendsignup container3 Error: {0}.format(e)')
        return {"url": f"{MY_HOME}dashboard/settings"}
        
    except exceptions.CosmosHttpResponseError as e:
        current_user["oid"]
        user = {
            "id": passport.email,
            "team": "yes",
            "email": db_result["email"],
            "oid": db_result["id"],
            "city": db_result["city"],
            "session_id": db_result["session_id"],
            "plan_id": db_result["plan_id"]
        }
        callConnect.container5.upsert_item(body=user)
        user2 = {
            "id": db_result2["id"],
            "members": db_result2["members"]
        }
        mems = db_result2["members"]

        mems.append({"name": passport.email})
        project = {
            "members": mems
        }
        user2.update(project)
        callConnect.container5.upsert_item(body=user2)
        TO_EMAILS = [(passport.email, passport.title)]
        message = Mail(from_email=FROM_EMAIL, to_emails=TO_EMAILS)
        message.dynamic_template_data = {
            'fund': passport.title,
        }
        message.template_id = TEMPLATE_ID3
        
        try:
            sg = SendGridAPIClient(SENDGRID_API_KEY)
            response = sg.send(message)
            code, body, headers = response.status_code, response.body, response.headers
            
        except Exception as e:
            logging.error('This is an error message for /sendsignup container3 Error: {0}.format(e)')
        return {"url": f"{MY_HOME}dashboard/settings"}




"""
Endpoint used to generate a SendGrid email to send to the company.
"""
@emailroute.post('/companyemail')
async def company_email(iCompany: InviteCompany2):
    TO_EMAILS = [(iCompany.email, iCompany.title)]
    message = Mail(from_email=FROM_EMAIL, to_emails=TO_EMAILS)
    message.dynamic_template_data = {
        'fund': iCompany.firm,
    }
    message.template_id = TEMPLATE_ID
    
    try:
        sg = SendGridAPIClient(SENDGRID_API_KEY)
        response = sg.send(message)
        code, body, headers = response.status_code, response.body, response.headers
        
    except Exception as e:
        logging.error('This is an error message for /companyemail Error: {0}.format(e)')
        
    return str(response.status_code)


"""
Endpoint used to generate a SendGrid email to request data from company. 
"""
@emailroute.post('/datarequestemail')
async def datarequest_email(iCompany: InviteCompany):
    try:
        db_result1 = read_item(callConnect.container3, iCompany.title, iCompany.companyemail)

    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /datarequestemail container3')
        return {"url": "/", "atype": False}
    
    if "approved" in db_result1:
        if ([d for d in db_result1["approved"] if d["id"] == iCompany.title]):
            if ([d for d in db_result1["approved"] for dd in d["funds"] if dd["name"] == iCompany.fund]):
                pass
        else:
            try:
                db_result = read_item(callConnect.container, iCompany.oid, iCompany.city)

            except exceptions.CosmosHttpResponseError as e:
                logging.error('This is an error message for /datarequestemail container')
                return {"url": "/", "atype": False}
            
            
            record = {
                "id": db_result1["id"],
                "email": db_result1["email"],
                "requested": db_result1["requested"],
                "approved": db_result1["approved"]
            }
            nreqs = db_result1["requested"]

            if ([d for d in db_result1["requested"] if d["fund"] == iCompany.fund]):
                pass
            else:
                nreqs.append({
                    "id": iCompany.oid,
                    "city": iCompany.city,
                    "name": iCompany.firm,
                    "email": iCompany.fundemail,
                    "fund": iCompany.fund
                })
            requested = {
                "requested": nreqs
            }
            record.update(requested)
            callConnect.container3.upsert_item(body=record)
            
            if "funds" in db_result:
                if ([d for d in db_result["funds"] for dd in d["pending"] if dd["name"] == iCompany.title]):
                    user2 = {
                        "id": db_result["id"],
                        "city": db_result["city"],
                        "email": db_result["email"],
                        "session_id": db_result["session_id"],
                        "sub_created": db_result["sub_created"],
                        "plan_id": db_result["plan_id"],
                        "funds": db_result["funds"],
                        "subs": db_result["subs"]
                    }
                else:
                    if ([d for d in db_result["funds"] for dd in d["companies"] if dd["id"]  == iCompany.title]):
                        user2 = {
                            "id": db_result["id"],
                            "city": db_result["city"],
                            "email": db_result["email"],
                            "session_id": db_result["session_id"],
                            "sub_created": db_result["sub_created"],
                            "plan_id": db_result["plan_id"],
                            "funds": db_result["funds"],
                            "subs": db_result["subs"]
                        }
                    else:
                        user2 = {
                            "id": db_result["id"],
                            "city": db_result["city"],
                            "email": db_result["email"],
                            "session_id": db_result["session_id"],
                            "sub_created": db_result["sub_created"],
                            "plan_id": db_result["plan_id"],
                            "funds": db_result["funds"],
                            "subs": db_result["subs"] - 1
                        }
                allfunds = [d for d in db_result["funds"] if d["fund"] != iCompany.fund]
                replacefund = [d for d in db_result["funds"] if d["fund"] == iCompany.fund][0]
                if "pending" in replacefund:
                    ccc = replacefund["companies"]
                    ppp = replacefund["pending"]
                    ppp = [p for p in replacefund["pending"] if p["name"] != iCompany.title]
                    ppp.append({"name": iCompany.title})
                    addfund = {
                        "fund": replacefund["fund"],
                        "fundstartdate": replacefund["fundstartdate"],
                        "companies": ccc,
                        "pending": ppp
                    }
                else:
                    ccc = replacefund["companies"]
                    ppp = []
                    ppp.append({"name": iCompany.title})
                    addfund = {
                        "fund": replacefund["fund"],
                        "fundstartdate": replacefund["fundstartdate"],
                        "companies": ccc,
                        "pending": ppp
                    }
                allfunds.append(addfund)
                fund = {
                    "funds": allfunds
                }
                user2.update(fund)
                callConnect.container.upsert_item(body=user2)

                TO_EMAILS = [(iCompany.companyemail, iCompany.title)]
                message = Mail(from_email=FROM_EMAIL, to_emails=TO_EMAILS)
                message.dynamic_template_data = {
                    'fund': iCompany.firm,
                }
                message.template_id = TEMPLATE_ID2
        
                TO_EMAILS2 = [("mmurday@metric-esg.com", iCompany.title)]
                message2 = Mail(from_email=FROM_EMAIL, to_emails=TO_EMAILS2)
                message2.dynamic_template_data = {
                    'fund': iCompany.firm,
                    'company': iCompany.title,
                    'company_email': iCompany.companyemail
                }
                message2.template_id = TEMPLATE_ID5
                try:
                    sg = SendGridAPIClient(SENDGRID_API_KEY)
                    response = sg.send(message)
                    response2 = sg.send(message2)
                    code, body, headers = response.status_code, response.body, response.headers
                    code2, body2, headers2 = response2.status_code, response2.body, response2.headers
                except Exception as e:
                    logging.error('This is an error message for /datarequestemail container3, Error: {0}.format(e)')

                return {"url": f"{MY_HOME}dashboard/fundoverview"}
    
    else:
        if "requested" in db_result1:
            record = {
                "id": db_result1["id"],
                "email": db_result1["email"],
                "requested": db_result1["requested"]
            }
            nreqs = db_result1["requested"]
            if ([d for d in db_result1["requested"] if d["fund"] == iCompany.fund]):
                pass
            else:
                nreqs.append({
                    "id": iCompany.oid,
                    "city": iCompany.city,
                    "name": iCompany.firm,
                    "email": iCompany.fundemail,
                    "fund": iCompany.fund
                })
            requested = {
                "requested": nreqs
            }
        else:
            record = {
                "id": db_result1["id"],
                "email": db_result1["email"]
            }
            requested = {
                "requested": [{"id": iCompany.oid, "city": iCompany.city, "name": iCompany.firm, "email": iCompany.fundemail, "fund": iCompany.fund}]
            }
        record.update(requested)
        callConnect.container3.upsert_item(body=record)

        try:
            db_result = read_item(callConnect.container, iCompany.oid, iCompany.city)


        except exceptions.CosmosHttpResponseError as e:
            logging.error('This is an error message for /datarequestemail container')
            return {"url": "/", "atype": False}
        

        if "funds" in db_result:
            if ([d for d in db_result["funds"] for dd in d["pending"] if dd["name"] == iCompany.title]):
                user2 = {
                    "id": db_result["id"],
                    "city": db_result["city"],
                    "email": db_result["email"],
                    "session_id": db_result["session_id"],
                    "sub_created": db_result["sub_created"],
                    "plan_id": db_result["plan_id"],
                    "funds": db_result["funds"],
                    "subs": db_result["subs"]
                }
            else:
                if ([d for d in db_result["funds"] for dd in d["companies"] if dd["id"]  == iCompany.title]):
                    user2 = {
                        "id": db_result["id"],
                        "city": db_result["city"],
                        "email": db_result["email"],
                        "session_id": db_result["session_id"],
                        "sub_created": db_result["sub_created"],
                        "plan_id": db_result["plan_id"],
                        "funds": db_result["funds"],
                        "subs": db_result["subs"]
                    }
                else:
                    user2 = {
                        "id": db_result["id"],
                        "city": db_result["city"],
                        "email": db_result["email"],
                        "session_id": db_result["session_id"],
                        "sub_created": db_result["sub_created"],
                        "plan_id": db_result["plan_id"],
                        "funds": db_result["funds"],
                        "subs": db_result["subs"] - 1
                    }
            allfunds = [d for d in db_result["funds"] if d["fund"] != iCompany.fund]
            replacefund = [d for d in db_result["funds"] if d["fund"] == iCompany.fund][0]

            if "pending" in replacefund:
                ppp = replacefund["pending"]
                if ([d for d in db_result["funds"] for dd in d["pending"] if dd["name"] == iCompany.title]):
                    pass
                else:
                    ppp.append({"name": iCompany.title})
                addfund = {
                    "fund": replacefund["fund"],
                    "fundstartdate": replacefund["fundstartdate"],
                    "companies": replacefund["companies"],
                    "pending": ppp
                }
            else:
                addfund = {
                    "fund": replacefund["fund"],
                    "fundstartdate": replacefund["fundstartdate"],
                    "companies": replacefund["companies"],
                    "pending": [{"name": iCompany.title}]
                }
            allfunds.append(addfund)
            fund = {
                "funds": allfunds
            }
        user2.update(fund)
        callConnect.container.upsert_item(body=user2)
        
        TO_EMAILS = [(iCompany.companyemail, iCompany.title)]
        message = Mail(from_email=FROM_EMAIL, to_emails=TO_EMAILS)
        message.dynamic_template_data = {
            'fund': iCompany.firm,
        }
        message.template_id = TEMPLATE_ID2

        TO_EMAILS2 = [("mmurday@metric-esg.com", iCompany.title)]
        message2 = Mail(from_email=FROM_EMAIL, to_emails=TO_EMAILS2)
        message2.dynamic_template_data = {
            'fund': iCompany.firm,
            'company': iCompany.title,
            'company_email': iCompany.companyemail
        }
        message2.template_id = TEMPLATE_ID5
        
        try:
            sg = SendGridAPIClient(SENDGRID_API_KEY)
            response = sg.send(message)
            response2 = sg.send(message2)
            
            code, body, headers = response.status_code, response.body, response.headers
            code2, body2, headers2 = response2.status_code, response2.body, response2.headers
        except Exception as e:
            logging.error('This is an error message for /datarequestemail container3, Error: {0}.format(e)')
        return {"url": f"{MY_HOME}dashboard/fundoverview"}



"""
Endpoint used to generate a SendGrid email to request data from company. 
"""
@emailroute.post('/datarequestemailNEW')
async def datarequest_email_new(iCompany: InviteCompany3):
    TO_EMAILS2 = [("mmurday@metric-esg.com", iCompany.title)]
    message2 = Mail(from_email=FROM_EMAIL, to_emails=TO_EMAILS2)
    message2.dynamic_template_data = {
        'fund': iCompany.fund,
        'company': iCompany.title
    }
    message2.template_id = TEMPLATE_ID5
    try:
        sg = SendGridAPIClient(SENDGRID_API_KEY)
        response2 = sg.send(message2)
        code2, body2, headers2 = response2.status_code, response2.body, response2.headers
    except Exception as e:
        logging.error('This is an error message for /datarequestemail container3, Error: {0}.format(e)')

    return {"url": f"{MY_HOME}dashboard/fundoverview"}



class Company(BaseModel):
    email: str
    name: str


"""
Endpoint used to send a sendgrid email for finalizing submission data
"""


@emailroute.post("/submission")
async def submit_data(company: Company):
    try:
        sg = SendGridAPIClient(SENDGRID_API_KEY)
        from_email = Email("info@metric-esg.com")
        to_email = To("analyst@metric-esg.com")

        html_content = f"""
        <html>
        <body>
            <p>Dear Analyst Team,</p>
            <p>Company: <strong>{company.name}</strong></p>
            <p>Email: <a href="mailto:{company.email}">{company.email}</a></p>
            <p>{company.name} has submitted data for review.</p>
            <p>Thank you</p>
            <p>Metric</p>
        </body>
        </html>
        """

        content = Content("text/html", html_content)

        subject = f"Request for Data Review from {company.name}"

        mail = Mail(from_email, to_email, subject, content)
        sg.send(mail)

        return {"message": "Email sent successfully!"}
    except Exception as e:
        logging.error(
            "This is an error message for /submission-email container3, Error: {0}.format(e)"
        )
        return {"message": "Email failed to send!"}
